/*
 * Copyright 2024 KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef XUNFEINLPENGINE_P_H
#define XUNFEINLPENGINE_P_H

#include <cpr/cpr.h>
#include <jsoncpp/json/json.h>
#include <kylin-ai/ai-engine/cloudnlpengine.h>
#include <kylin-ai/ai-engine/error.h>

class XunfeiNlpEnginePrivate : public ai_engine::nlp::AbstractCloudNlpEngine {
public:
    XunfeiNlpEnginePrivate();
    ~XunfeiNlpEnginePrivate() override;
    std::string engineName() const override;
    std::string currentModelName() const override;
    int maxConcurrentChatTasks() const override;

    void setConfig(const std::string &config) override;
    std::string modelInfo() const override;

    void
    setChatResultCallback(ai_engine::nlp::ChatResultCallback callback) override;

    void setContextSize(int size) override;
    void clearContext() override;

    bool initChatModule(ai_engine::EngineError &error) override;
    bool chat(const std::string &message,
              ai_engine::EngineError &error) override;
    void stopChat() override;
    bool destroyChatModule(ai_engine::EngineError &error) override;

private:
    void threadSafeCallChatResultCallback(
        const ai_engine::nlp::ChatResult &result) const;

    void handleReceiveData();
    std::string receiveChatData(int *retCode) const;
    bool isNetworkError(int retCode) const;
    bool isChatDataReady(int retCode) const;
    bool needReReceiveData(int retCode) const;

    void handleNetworkError(int retCode);
    void handleServerError(int errorCode, const std::string &errorMessage);
    void handleChatData(const std::string &chatData, bool *isFinished);

    CURLcode testConnectToServer();

private:
    std::string appId_;
    std::string apiKey_;
    std::string secretKey_;
    std::string hostUrl_{"wss://spark-api.xf-yun.com/v3.5/chat"};

    mutable std::mutex mutex_;
    ai_engine::nlp::ChatResultCallback chatResultCallback_;

    Json::Value context_;
    void addContext(const std::string &role, const std::string &message);
    void removeLastContext();
    std::vector<Json::Value> streamChatData_;

    bool connectWebsocket(const std::string &hostUrl);
    friend void onWebSocket(bool (*doReceiveData)(XunfeiNlpEnginePrivate *self),
                            XunfeiNlpEnginePrivate *self);
    void closeWebsocket();

    CURL *currentCurl_{nullptr};
    ai_engine::EngineError currentError_;
    bool currentStatus();

    bool initResources();
    void releaseResources();

    bool doSendData(const char *data, int dataLength, int type);

    std::atomic_bool isStopped_{false};

    static const std::string systemRole_;
};

#endif // XUNFEINLPENGINE_P_H
